﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ArmoredCar
{
    internal class SetArmoredCarsGeneric<T>
        where T : class
    {
        /// <summary>
        /// Список объектов, которые храним
        /// </summary>
        private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Count;
        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetArmoredCarsGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="armoredCar">Добавляемый автомобиль</param>
        /// <returns></returns>
        public int Insert(T armoredCar)
        {   
            if (Count < _maxCount)
                return Insert(armoredCar, 0);
            return -1;
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="armoredCar">Добавляемый автомобиль</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T armoredCar, int position)
        {
            if (position < 0 || position >= _maxCount)
                return -1;

            _places.Insert(position, armoredCar);         
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position < 0 ||  position >= Count)
                return null;
            T armoredCar = _places[position];
            _places.RemoveAt(position);
            return armoredCar;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T this[int position]
        {
            get
            {
                if (position < 0 || position >= Count)
                    return null;
                return _places[position];
            }
            set
            {
                
                if (position < 0 || position >= _maxCount)
                    return;
                
                _places.Add(value);
            }
        }
        /// <summary>
        /// Проход по набору до первого пустого
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T> GetCars()
        {
            foreach (var car in _places)
            {
                if (car != null)
                {
                    yield return car;
                }
                else
                {
                    yield break;
                }
            }
        }

    }
}
